//
//  WiFiSharingConnection.m
//  Voice Recorder
//
//  Created by Linfei on 2020/1/19.
//  Copyright © 2020 Linfei Ltd. All rights reserved.
//

#import "WiFiSharingConnection.h"
#import "TimelineDataSource.h"
#import "AKFileManager.h"
#import "HTTPResponse.h"
#import "HTTPFileResponse.h"
#import "HTTPAsyncFileResponse.h"
#import "HTTPDataResponse.h"
#import "Utilities.h"
#import "Actions.h"

@interface HTTPSvgFileResponse : HTTPFileResponse
@end

@implementation HTTPSvgFileResponse

- (NSDictionary *)httpHeaders
{
    NSMutableDictionary *header = [NSMutableDictionary dictionary];
    [header setObject:@"image/svg+xml;" forKey:@"content-type"];
    
    return header;
}

@end

@implementation WiFiSharingConnection

- (NSString *)createBrowseableIndex
{
    [Actions shared].wifiShared = YES;
    
    NSError *error = nil;
    NSRange range;
    range.location = 0;
    
    NSArray<AKFile *> *allFiles = nil;
    if([Actions shared].wifiSharingFiles)
    {
        allFiles = [Actions shared].wifiSharingFiles;
    }
    else
    {
        TimelineDataSource *dataSource = [[TimelineDataSource alloc] initWithFolders:[AKFileManager shared].availableFolders];
        allFiles = [dataSource generateAllFiles];
    }
    NSMutableString *content = [NSMutableString new];
    NSMutableString *rows = [NSMutableString new];
    error = nil;
    NSString *rowFormat = [NSMutableString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"row" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error].mutableCopy;
    if(error) NSLog(@"%@", error.localizedDescription);
    for (AKFile *file in allFiles)
    {
        NSMutableString *row = rowFormat.mutableCopy;
        
        range.length = [row length];
        [row replaceOccurrencesOfString:@"%Name%" withString:file.displayName options:NSCaseInsensitiveSearch range:range];
        
        range.length = [row length];
        [row replaceOccurrencesOfString:@"%Ext%" withString:file.extension options:NSCaseInsensitiveSearch range:range];
        
        range.length = [row length];
        [row replaceOccurrencesOfString:@"%Duration%" withString:file.displayDuration options:NSCaseInsensitiveSearch range:range];
        
        NSString *fileName = file.relativePath;
        if(file.baseLocation == AKBaseLocationLinFeiCloud) fileName = [file.fileID stringByAppendingPathExtension:file.extension];
        NSArray<NSString *> *components = [[NSString stringWithFormat:@"d/%lu/%@", (unsigned long)file.baseLocation, fileName] componentsSeparatedByString:@"/"];
        NSMutableString *path = [NSMutableString new];
        NSInteger count = components.count;
        for (NSInteger i = 0; i < count; i++)
        {
            if(i == (count - 1))
            {
                NSString *name = [components[i] stringByDeletingPathExtension];
                NSString *extension = components[i].pathExtension;
                [path appendFormat:@"%@.%@", [Utilities urlEncoding:name], extension];
            }
            else
            {
                [path appendFormat:@"%@/", [Utilities urlEncoding:components[i]]];
            }
        }
        
        range.length = [row length];
        [row replaceOccurrencesOfString:@"%Download%" withString:path options:NSCaseInsensitiveSearch range:range];
        
        [rows appendString:row];
    }
    if(rows.length > 0)
    {
        error = nil;
        content = [NSMutableString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"content" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error].mutableCopy;
        if(error) NSLog(@"%@", error.localizedDescription);
        
        range.length = [content length];
        [content replaceOccurrencesOfString:@"%Rows%" withString:rows options:NSCaseInsensitiveSearch range:range];
    }
    
    error = nil;
    NSMutableString *templateString = [NSMutableString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"index" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
    if(error) NSLog(@"%@", error.localizedDescription);
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Title%" withString:NSLocalizedString(@"WifiSharing", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%App%" withString:[NSBundle mainBundle].localizedInfoDictionary[@"CFBundleName"] options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Content%" withString:content options:NSCaseInsensitiveSearch range:range];
    
    return templateString;
}

- (NSObject<HTTPResponse> *)httpResponseForMethod:(NSString *)method URI:(NSString *)path
{
    path = [Utilities urlDecoding:path];

    if([path isEqualToString:@"/"])
    {
        NSData *data = [[self createBrowseableIndex] dataUsingEncoding:NSUTF8StringEncoding];
        return [[HTTPDataResponse alloc] initWithData:data];
    }
    else if(([path.pathExtension.lowercaseString isEqualToString:@"svg"]))
    {
        NSString *filename = [path.lastPathComponent stringByDeletingPathExtension];
        NSString *extension = path.pathExtension;

        return [[HTTPSvgFileResponse alloc] initWithFilePath:[[NSBundle mainBundle] pathForResource:filename ofType:extension] forConnection:self];
    }
    else
    {
        NSArray<NSString *> *components = [path componentsSeparatedByString:@"/"];
        if(components.count < 4) return nil;
        
        NSString *protocol = components[1];
        if([protocol isEqualToString:@"d"])
        {
            AKBaseLocation baseLocation = (AKBaseLocation)[components[2] integerValue];
            NSMutableString *relativePath = [NSMutableString new];
            for (NSInteger i = 3; i < components.count; i++)
            {
                if(i > 3)
                {
                    [relativePath appendString:@"/"];
                }
                [relativePath appendString:components[i]];
            }
            
            NSURL *fileURL;
            NSString *fileName;
            if(baseLocation == AKBaseLocationLinFeiCloud)
            {
                NSString *fileID = [relativePath stringByDeletingPathExtension];
                if(@available(iOS 13, *))
                {
                    AKOSSFile *file = [[AKFileManager shared] findFile:fileID];
                    if(file)
                    {
                        fileURL = file.fileURL;
                        fileName = [[Utilities urlEncoding:file.fileName.stringByDeletingPathExtension] stringByAppendingPathExtension:file.fileName.pathExtension];
                    }
                }
            }
            else
            {
                fileURL = [[AKFileManager shared] fileURLWithLocation:baseLocation relative:relativePath];
            }
            
            if(fileURL && [[NSFileManager defaultManager] fileExistsAtPath:fileURL.path])
            {
                HTTPFileResponse *response = [[HTTPFileResponse alloc] initWithFilePath:fileURL.path forConnection:self fileName:fileName];
                return response;
            }
        }
    }
    
    return nil;
}

@end
