//
//  WiFiSharingConnection.m
//  Voice Recorder
//
//  Created by Linfei on 2020/1/19.
//  Copyright © 2020 Linfei Ltd. All rights reserved.
//

#import "WiFiSharingTranscreibeConnection.h"
#import "TimelineDataSource.h"
#import "AKFileManager.h"
#import "HTTPResponse.h"
#import "HTTPFileResponse.h"
#import "HTTPAsyncFileResponse.h"
#import "HTTPDataResponse.h"
#import "Utilities.h"
#import "Actions.h"

@interface HTTPSvgFileResponse2 : HTTPFileResponse
@end

@implementation HTTPSvgFileResponse2

- (NSDictionary *)httpHeaders
{
    NSMutableDictionary *header = [NSMutableDictionary dictionary];
    [header setObject:@"image/svg+xml;" forKey:@"content-type"];
    
    return header;
}

@end

@interface HTTPJSFileResponse : HTTPFileResponse
@end

@implementation HTTPJSFileResponse

- (NSDictionary *)httpHeaders
{
    NSMutableDictionary *header = [NSMutableDictionary dictionary];
    [header setObject:@"application/x-javascript;" forKey:@"content-type"];
    
    return header;
}

@end

@implementation WiFiSharingTranscreibeConnection

- (NSString *)createBrowseableIndex
{
    [Actions shared].wifiShared = YES;
    
    NSError *error = nil;
    NSRange range;
    range.location = 0;
    
    NSArray<AKFile *> *allFiles = nil;
    int shareTextMode = [Actions shared].wifiSharingType;
    if([Actions shared].wifiSharingFiles)
    {
        allFiles = [Actions shared].wifiSharingFiles;
    }
    else
    {
        TimelineDataSource *dataSource = [[TimelineDataSource alloc] initWithFolders:[AKFileManager shared].availableFolders];
        allFiles = [dataSource generateAllFiles];
    }

    error = nil;
    
    AKFile *file = allFiles.firstObject;
    
    NSMutableString *templateString = [NSMutableString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"transcreibe" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
    if(error) NSLog(@"%@", error.localizedDescription);
    
    NSString *strText = @"";
    NSString *strNote = @"";
    NSString *strTimeline = @"";
    BOOL showText = file.existingText;
    BOOL showTimeline = file.existingTranscript || file.existingTimeline;
    BOOL showNote = file.existingNote;
    if(showText) strText = [file.readText stringByReplacingOccurrencesOfString:@"\n" withString:@"<br />"];
    if(showNote) strNote = [file.readNote stringByReplacingOccurrencesOfString:@"\n" withString:@"<br />"];
    if(showTimeline) strTimeline = [file.readTranscript stringByReplacingOccurrencesOfString:@"\n" withString:@"<br />"];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Title%" withString:NSLocalizedString(@"WifiSharing", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%App%" withString:[NSBundle mainBundle].localizedInfoDictionary[@"CFBundleName"] options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Copy%" withString:NSLocalizedString(@"Copy", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Save%" withString:NSLocalizedString(@"Save", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Saved%" withString:NSLocalizedString(@"CopiedToBoard", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Fail%" withString:NSLocalizedString(@"Failed", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%TimeLineTitle%" withString:NSLocalizedString(@"ASR_Timeline", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%TextTitle%" withString:NSLocalizedString(@"ASR_Transcribe_Text", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%NoteTitle%" withString:NSLocalizedString(@"Text", nil) options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%TimeLine%" withString:strTimeline options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Text%" withString:strText options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%Note%" withString:strNote options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%TimeLineClass%" withString:(showTimeline?(shareTextMode == 1?@"tab_sel":@"tab"):@"tab_hid") options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%TextClass%" withString:(showText?(shareTextMode == 2?@"tab_sel":@"tab"):@"tab_hid") options:NSCaseInsensitiveSearch range:range];
    
    range.length = [templateString length];
    [templateString replaceOccurrencesOfString:@"%NoteClass%" withString:(showNote?(shareTextMode == 3?@"tab_sel":@"tab"):@"tab_hid") options:NSCaseInsensitiveSearch range:range];
    
    return templateString;
}

- (NSObject<HTTPResponse> *)httpResponseForMethod:(NSString *)method URI:(NSString *)path
{
    path = [Utilities urlDecoding:path];

    if([path isEqualToString:@"/"])
    {
        NSData *data = [[self createBrowseableIndex] dataUsingEncoding:NSUTF8StringEncoding];
        return [[HTTPDataResponse alloc] initWithData:data];
    }
    else if(([path.pathExtension.lowercaseString isEqualToString:@"js"]))
    {
        return [[HTTPJSFileResponse alloc] initWithFilePath:[[NSBundle mainBundle] pathForResource:@"clipboard.min" ofType:@"js"] forConnection:self];
    }
    else if(([path.pathExtension.lowercaseString isEqualToString:@"svg"]))
    {
        NSString *filename = [path.lastPathComponent stringByDeletingPathExtension];
        NSString *extension = path.pathExtension;

        return [[HTTPSvgFileResponse2 alloc] initWithFilePath:[[NSBundle mainBundle] pathForResource:filename ofType:extension] forConnection:self];
    }
    else
    {
        NSArray<NSString *> *components = [path componentsSeparatedByString:@"/"];
        if(components.count < 4) return nil;
        
        NSString *protocol = components[1];
        if([protocol isEqualToString:@"d"])
        {
            AKBaseLocation baseLocation = (AKBaseLocation)[components[2] integerValue];
            NSMutableString *relativePath = [NSMutableString new];
            for (NSInteger i = 3; i < components.count; i++)
            {
                if(i > 3)
                {
                    [relativePath appendString:@"/"];
                }
                [relativePath appendString:components[i]];
            }
            
            NSURL *fileURL;
            NSString *fileName;
            if(baseLocation == AKBaseLocationLinFeiCloud)
            {
                NSString *fileID = [relativePath stringByDeletingPathExtension];
                if(@available(iOS 13, *))
                {
                    AKOSSFile *file = [[AKFileManager shared] findFile:fileID];
                    if(file)
                    {
                        fileURL = file.fileURL;
                        fileName = [[Utilities urlEncoding:file.fileName.stringByDeletingPathExtension] stringByAppendingPathExtension:file.fileName.pathExtension];
                    }
                }
            }
            else
            {
                fileURL = [[AKFileManager shared] fileURLWithLocation:baseLocation relative:relativePath];
            }
            
            if(fileURL && [[NSFileManager defaultManager] fileExistsAtPath:fileURL.path])
            {
                HTTPFileResponse *response = [[HTTPFileResponse alloc] initWithFilePath:fileURL.path forConnection:self fileName:fileName];
                return response;
            }
        }
    }
    
    return nil;
}

@end
